@@ -21,41 +21,25 @@
 #include <QtGui>
 #include <QtNetwork>
 
-static QString getSynergyVersion(const QString& app)
-{
-	QProcess process;
-	process.start(app, QStringList() << "--version");
-
-	process.setReadChannel(QProcess::StandardOutput);
-	if (process.waitForStarted() && process.waitForFinished())
-	{
-		QRegExp rx("synergy[cs] ([\\d\\.]+)");
-		if (rx.indexIn(process.readLine()) != -1)
-			return rx.cap(1);
-	}
-
-	return "Unknown";
-}
-
-static QString getIPAddress()
-{
-	QList<QHostAddress> addresses = QNetworkInterface::allAddresses();
-
-	for (int i = 0; i < addresses.size(); i++)
-		if (addresses[i].protocol() == QAbstractSocket::IPv4Protocol && addresses[i] != QHostAddress(QHostAddress::LocalHost))
-			return addresses[i].toString();
-
-	return "Unknown";
-}
-
 AboutDialog::AboutDialog(QWidget* parent, const QString& synergyApp) :
 	QDialog(parent, Qt::WindowTitleHint | Qt::WindowSystemMenuHint),
 	Ui::AboutDialogBase()
 {
 	setupUi(this);
 
-	m_pLabelSynergyVersion->setText(getSynergyVersion(synergyApp));
-	m_pLabelHostname->setText(QHostInfo::localHostName());
-	m_pLabelIPAddress->setText(getIPAddress());
-}
+	m_versionChecker.setApp(synergyApp);
+	m_pLabelSynergyVersion->setText(m_versionChecker.getVersion());
 
+    // change default size based on os
+#if defined(Q_OS_MAC)
+	QSize size(600, 350);
+	setMaximumSize(size);
+	setMinimumSize(size);
+	resize(size);
+#elif defined(Q_OS_LINUX)
+	QSize size(600, 300);
+	setMaximumSize(size);
+	setMinimumSize(size);
+	resize(size);
+#endif
+}
