@@ -31,6 +31,8 @@ public:
 	// IArchSystem overrides
 	virtual std::string	getOSName() const;
 	virtual std::string getPlatformName() const;
+	virtual std::string setting(const std::string& valueName) const;
+	virtual void setting(const std::string& valueName, const std::string& valueString) const;
 
 	bool isWOW64() const;
 };
