@@ -32,6 +32,8 @@
 #include "CEventQueue.h"
 #include "LogOutputters.h"
 #include "CFunctionEventJob.h"
+#include "TMethodJob.h"
+#include "CVncClient.h"
 
 #if SYSAPI_WIN32
 #include "CArchMiscWindows.h"
@@ -62,7 +64,8 @@ s_serverState(kUninitialized),
 s_serverScreen(NULL),
 s_primaryClient(NULL),
 s_listener(NULL),
-s_timer(NULL)
+s_timer(NULL),
+m_vncClient(NULL)
 {
 }
 
@@ -320,6 +323,13 @@ CServerApp::handleClientConnected(const CEvent&, void* vlistener)
 	if (client != NULL) {
 		s_server->adoptClient(client);
 		updateStatus();
+
+		if (args().m_enableVnc) {
+			// TODO: figure out client IP address from name.
+			CVncClient* vncClient = new CVncClient("192.168.0.13", client->getName());
+			vncClient->start();
+			m_vncClients.insert(std::pair<CString, CVncClient*>(client->getName(), vncClient));
+		}
 	}
 }
 
@@ -346,13 +356,9 @@ CServerApp::closeServer(CServer* server)
 		new TMethodEventJob<CServerApp>(this, &CServerApp::handleClientsDisconnected));
 	EVENTQUEUE->adoptHandler(CServer::getDisconnectedEvent(), server,
 		new TMethodEventJob<CServerApp>(this, &CServerApp::handleClientsDisconnected));
-	CEvent event;
-	EVENTQUEUE->getEvent(event);
-	while (event.getType() != CEvent::kQuit) {
-		EVENTQUEUE->dispatchEvent(event);
-		CEvent::deleteData(event);
-		EVENTQUEUE->getEvent(event);
-	}
+	
+	EVENTQUEUE->loop();
+
 	EVENTQUEUE->removeHandler(CEvent::kTimer, timer);
 	EVENTQUEUE->deleteTimer(timer);
 	EVENTQUEUE->removeHandler(CServer::getDisconnectedEvent(), server);
@@ -639,7 +645,8 @@ CScreen*
 CServerApp::createScreen()
 {
 #if WINAPI_MSWINDOWS
-	return new CScreen(new CMSWindowsScreen(true, args().m_noHooks, args().m_gameDevice));
+	return new CScreen(new CMSWindowsScreen(
+		true, args().m_noHooks, args().m_gameDevice, args().m_stopOnDeskSwitch));
 #elif WINAPI_XWINDOWS
 	return new CScreen(new CXWindowsScreen(
 		args().m_display, true, args().m_disableXInitThreads, 0, *EVENTQUEUE));
@@ -704,6 +711,10 @@ CServerApp::openServer(const CConfig& config, CPrimaryClient* primaryClient)
 			CServer::getDisconnectedEvent(), server,
 			new TMethodEventJob<CServerApp>(this, &CServerApp::handleNoClients));
 
+		EVENTQUEUE->adoptHandler(
+			CServer::getScreenSwitchedEvent(), server,
+			new TMethodEventJob<CServerApp>(this, &CServerApp::handleScreenSwitched));
+
 	} catch (std::bad_alloc &ba) {
 		delete server;
 		throw ba;
@@ -712,20 +723,42 @@ CServerApp::openServer(const CConfig& config, CPrimaryClient* primaryClient)
 	return server;
 }
 
-void CServerApp::handleNoClients( const CEvent&, void* )
+void
+CServerApp::handleNoClients(const CEvent&, void*)
 {
 	updateStatus();
 }
 
-int CServerApp::mainLoop()
+void
+CServerApp::handleScreenSwitched(const CEvent& e, void*)
+{
+	if (!args().m_enableVnc)
+		return;
+
+	if (m_vncClient != NULL) {
+		LOG((CLOG_DEBUG "hiding vnc viewer for: %s", m_vncClient->m_screen.c_str()));
+		m_vncClient->hideViewer();
+	}
+
+	CServer::CSwitchToScreenInfo* info = reinterpret_cast<CServer::CSwitchToScreenInfo*>(e.getData());
+	std::map<CString, CVncClient*>::iterator it = m_vncClients.find(info->m_screen);
+	if (it == m_vncClients.end()) {
+		LOG((CLOG_DEBUG "could not find vnc client for: %s", info->m_screen));
+		return;
+	}
+
+	LOG((CLOG_DEBUG "showing vnc viewer for: %s", info->m_screen));
+	m_vncClient = it->second;
+	m_vncClient->showViewer();
+}
+
+int
+CServerApp::mainLoop()
 {
 	// create socket multiplexer.  this must happen after daemonization
 	// on unix because threads evaporate across a fork().
 	CSocketMultiplexer multiplexer;
 
-	// create the event queue
-	CEventQueue eventQueue;
-
 	// if configuration has no screens then add this system
 	// as the default
 	if (args().m_config->begin() == args().m_config->end()) {
@@ -750,7 +783,16 @@ int CServerApp::mainLoop()
 	}
 
 	// start server, etc
-	ARCH->util().startNode();
+	appUtil().startNode();
+	
+	// init ipc client after node start, since create a new screen wipes out
+	// the event queue (the screen ctors call adoptBuffer).
+	if (argsBase().m_enableIpc) {
+		initIpcClient();
+	}
+
+	// load all available plugins.
+	ARCH->plugin().init(s_serverScreen->getEventTarget());
 
 	// handle hangup signal by reloading the server's configuration
 	ARCH->setSignalHandler(CArch::kHANGUP, &reloadSignalHandler, NULL);
@@ -773,14 +815,8 @@ int CServerApp::mainLoop()
 	// run event loop.  if startServer() failed we're supposed to retry
 	// later.  the timer installed by startServer() will take care of
 	// that.
-	CEvent event;
 	DAEMON_RUNNING(true);
-	EVENTQUEUE->getEvent(event);
-	while (event.getType() != CEvent::kQuit) {
-		EVENTQUEUE->dispatchEvent(event);
-		CEvent::deleteData(event);
-		EVENTQUEUE->getEvent(event);
-	}
+	EVENTQUEUE->loop();
 	DAEMON_RUNNING(false);
 
 	// close down
@@ -793,6 +829,10 @@ int CServerApp::mainLoop()
 	updateStatus();
 	LOG((CLOG_NOTE "stopped server"));
 
+	if (argsBase().m_enableIpc) {
+		cleanupIpcClient();
+	}
+
 	return kExitSuccess;
 }
 
